/*******************************************************************************
 * Copyright (c) 2004, 2005 Elias Volanakis and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Elias Volanakis - initial API and implementation
 *******************************************************************************/
package jp.hasc.hasctool.ui.bdeditor2.model;

import jp.hasc.hasctool.core.blockdiagram.model.BeanBlock;
import jp.hasc.hasctool.core.blockdiagram.model.Connection;
import jp.hasc.hasctool.core.blockdiagram.model.PortReference;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

/**
 * A connection between two distinct shapes.
 * 
 * @author Elias Volanakis
 */
public class ConnectionElement extends ModelElement {
	/** logger for this class */
	private final static org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory
			.getLog(ConnectionElement.class);

	private static final long serialVersionUID = 1;

	/** True, if the connection is attached to its endpoints. */
	private boolean isConnected;
	/** Connection's source endpoint. */
	private BlockElement source;
	/** Connection's target endpoint. */
	private BlockElement target;
	
	public ConnectionElement() {
		
	}
	
	public void init(BlockDiagramElement bdg, BlockElement source, BlockElement target) {
		init(bdg,source,target,new Connection(
				new PortReference(source.getBeanBlock().getName(), PortReference.OUTPUT_PORT),
				new PortReference(target.getBeanBlock().getName(), PortReference.INPUT_PORT)));
	}
	
	public boolean init(BlockDiagramElement bdg, Connection c) {
		BlockElement source = bdg.getBlockObjMap().get(c.getOutputPortReference().getBlockName());
		BlockElement target = bdg.getBlockObjMap().get(c.getInputPortReference().getBlockName());
		if (source==null || target==null) return false;
		init(bdg,source,target,c);
		return true;
	}
	
	private void init(BlockDiagramElement bdg, BlockElement source, BlockElement target, Connection c) {
		bdg_ = bdg;
		this.source=source;
		this.target=target;
		connection_ = c;
		reconnect();
	}

	/**
	 * Disconnect this connection from the shapes it is attached to.
	 */
	public void disconnect() {
		if (isConnected) {
			source.removeConnection(this);
			target.removeConnection(this);
			bdg_.removeConnection(this);
			isConnected = false;
		}
	}
	
	/**
	 * Returns the source endpoint of this connection.
	 * 
	 * @return a non-null Shape instance
	 */
	public BlockElement getSource() {
		return source;
	}

	/**
	 * Returns the target endpoint of this connection.
	 * 
	 * @return a non-null Shape instance
	 */
	public BlockElement getTarget() {
		return target;
	}

	/**
	 * Reconnect this connection. The connection will reconnect with the shapes
	 * it was previously attached to.
	 */
	public void reconnect() {
		if (!isConnected) {
			source.addConnection(this);
			target.addConnection(this);
			bdg_.addConnection(this);
			isConnected = true;
		}
	}

	/**
	 * Reconnect to a different source and/or target shape. The connection will
	 * disconnect from its current attachments and reconnect to the new source
	 * and target.
	 * 
	 * @param newSource
	 *            a new source endpoint for this connection (non null)
	 * @param newTarget
	 *            a new target endpoint for this connection (non null)
	 * @throws IllegalArgumentException
	 *             if any of the paramers are null or newSource == newTarget
	 */
	public void reconnect(BlockElement newSource, BlockElement newTarget) {
		disconnect();
		connection_.setInputPortReference(new PortReference(newSource.getBeanBlock().getName(), 
				connection_.getInputPortReference().getPortName()));
		connection_.setOutputPortReference(new PortReference(newTarget.getBeanBlock().getName(), 
				connection_.getOutputPortReference().getPortName()));
		reconnect();
	}
	
	@Override
	public String toString() {
		return "Connection " + hashCode();
	}
	
	
	//////
	
	private BlockDiagramElement bdg_;
	private Connection connection_;
	
	@Override
	public Object getEditableValue() {
		return null;
	}

	private final static String P_TYPE = "P_TYPE";
	private final static String P_COMMENT = "P_COMMENT";
	private final static String P_OUTPUT_PORT = "P_OUTPUT";
	private final static String P_INPUT_PORT = "P_INPUT";

	@Override
	public IPropertyDescriptor[] getPropertyDescriptors() {
		IPropertyDescriptor[] descriptor = new IPropertyDescriptor[]{
				new TextPropertyDescriptor(P_TYPE,"_Type"),
				new TextPropertyDescriptor(P_COMMENT,"Comment"),
				new TextPropertyDescriptor(P_OUTPUT_PORT,"OutputPort"),
				new TextPropertyDescriptor(P_INPUT_PORT,"InputPort")
		};
		return descriptor;
	}

	@Override
	public Object getPropertyValue(Object id) {
		if(P_TYPE.equals(id)){
			return "Connection";
		}else if(P_COMMENT.equals(id)){
			return connection_.getComment()!=null?connection_.getComment():"";
		}else if(P_OUTPUT_PORT.equals(id)){
			return connection_.getOutputPortReference().getPortName();
		}else if(P_INPUT_PORT.equals(id)){
			return connection_.getInputPortReference().getPortName();
		}
		return null;
	}

	@Override
	public boolean isPropertySet(Object id) {
		return false;
	}

	@Override
	public void resetPropertyValue(Object id) {
	}

	@Override
	public void setPropertyValue(Object id, Object value) {
		LOG.debug("setPropertyValue: "+id+"="+value);
		
		String valueStr=value!=null?value.toString():"";
		if(P_COMMENT.equals(id)){
			connection_.setComment(valueStr.isEmpty()?null:valueStr);
			bdg_.updateText();
		}else if(P_OUTPUT_PORT.equals(id)){
			if (!valueStr.isEmpty()) {
				BeanBlock blk = bdg_.getBlockObjMap().get(connection_.getOutputPortReference().getBlockName()).getBeanBlock();
				connection_.setOutputPortReference(blk.getPortByName(valueStr));
				bdg_.updateText();
			}
		}else if(P_INPUT_PORT.equals(id)){
			if (!valueStr.isEmpty()) {
				BeanBlock blk = bdg_.getBlockObjMap().get(connection_.getInputPortReference().getBlockName()).getBeanBlock();
				connection_.setInputPortReference(blk.getPortByName(valueStr));
				bdg_.updateText();
			}
		}
	}

	public Connection getConnection() {
		return connection_;
	}
	
}